% direct_fit        - Fit measured open-loop response

function data = direct_fit(data, span);


data.w = 2*pi*data.freq;
frf = data.f_if + data.f_lo;

f = data.freq -frf;
data.u = find(f>=min(span) & f<=max(span));

% Compute interpolator response
theta = 2*pi*data.f_if/data.f_s;
C = 1/2/cos(theta/2); % Interpolator coefficient
w0 = (data.freq - data.f_lo)/data.f_s * pi;
data.H_inter = freqz([C 1 C], [0 2], w0);

% Compute rotator response
if isfield(data, 'rot')
  data.vec = rot(data.rot.gn, data.rot.ph, theta);
  data.H_rot = freqz(data.vec, 1, w0);
end

% First, fit the magnitude, extracting G, sigma, wr
[G0, idx] = max(abs(data.H(data.u)));
wr0       = data.w(data.u(idx(1)));
ph0       = data.phase(data.u(idx(1)));
u         = find(max(data.mag(data.u))-data.mag(data.u)<3);
sigma0    = (data.w(data.u(max(u)))-data.w(data.u(min(u))));

x0 = [G0 sigma0 wr0];

opts = optimset('TolX', 1e-9, 'TolFun', 1e-9, 'Disp', 'none');
x = fminsearch('direct_fit_mag', x0, opts, data);

data.fit.G     = x(1);
data.fit.sigma = x(2);
data.fit.wr    = x(3);
data.fit.x     = x;

x0 = [500e-9 ph0];

x = fminsearch('direct_fit_ph', x0, opts, data);

data.fit.tau   = x(1);
data.fit.phi   = mod(x(2), 2*pi)/pi*180;
data.fit.x     = [data.fit.x x];

% Finally, run joint optimization
x0 = data.fit.x;
x  = fminsearch('direct_fit_all', x0, opts, data);

data.fit.G     = x(1);
data.fit.sigma = x(2);
data.fit.wr    = x(3);
data.fit.tau   = x(4);
data.fit.phi   = mod(x(5), 2*pi)/pi*180;
data.fit.Q     = x(3)/x(2);
data.fit.x     = x;

data.u = 1:length(f);

H = direct_ol(data.fit.x, data);

clf;
subplot(211); set(gca, 'fontsize', 12);
h = plot((data.freq-frf)/1e3, [data.mag; 20*log10(abs(H))]);
set(h(1), 'linewidth', 2);
set(h(2), 'color', 'r');
xlabel('Frequency offset (kHz)');
ylabel('Gain (dB)');
legend('Data', 'Fit');
title(sprintf('Gain = %.1f, Q = %g, (w_r - w_{rf}) = %.2f kHz', ...
      data.fit.G, data.fit.Q, (data.fit.wr - data.wrf)/2e3/pi));
grid;

subplot(212); set(gca, 'fontsize', 12);
h = plot((data.freq-frf)/1e3, angle([data.H; H])/pi*180);
set(h(1), 'linewidth', 2);
set(h(2), 'color', 'r');
xlabel('Frequency offset (kHz)');
ylabel('Phase (degrees)');
legend('Data', 'Fit');
title(sprintf('\\tau = %g ns, \\phi = %.1f deg', data.fit.tau*1e9, data.fit.phi));
grid;
